import os
import sys
import time
import logging
import argparse

from pymygdala import LogHandler
from pycapo import CapoConfig
from ._version import ___version___ as version

_DESCRIPTION = """Workspaces null executable, a status capture test of the system. Version {}"""

# logging.basicConfig()
logger = logging.getLogger("null")
logger.setLevel(logging.INFO)
config = CapoConfig()

class Null:
    def __init__(self, args, verbose):
        self.args = args
        if verbose:
            logger.setLevel(logging.DEBUG)
        self.args_to_funcs = {
            'greeting': self.print_greeting,
            'exit': self.exit_with_failure,
            'nap': self.take_nap,
            'dump': self.dump_core
        }

    def print_greeting(self):
        logger.info("Hello, world!")
        logger.debug("And goodbye, world...")

    def exit_with_failure(self):
        logger.error("Error purposefully induced.")
        sys.exit('Exiting with status code -1')

    def take_nap(self):
        logger.debug("Going to sleep...")
        time.sleep(5)
        logger.debug("Waking up.")

    def dump_core(self):
        logger.debug("Aborting and dumping core...", stack_info=True)
        os.abort()

    def execute(self):
        for arg, val in vars(self.args).items():
            if val and arg in self.args_to_funcs:
                self.args_to_funcs[arg]()

def make_arg_parser():
    parser = argparse.ArgumentParser(description=_DESCRIPTION.format(version),
                                     formatter_class=argparse.RawTextHelpFormatter)
    options = parser.add_argument_group('options', 'settings for altering program behavior')
    options.add_argument('-v', '--verbose', action='store_true',
                         required=False, dest='verbose', default=False,
                         help='allow the program the gift of speech')
    options.add_argument('-P', '--profile', action='store',
                         required=True, dest='profile', default=False,
                         help='profile name to use, e.g. test, production')
    functions = parser.add_mutually_exclusive_group(required=False)
    functions.add_argument('-g', '--greeting', action='store_true',
                           required=False, dest='greeting', default=False,
                           help='print out a friendly greeting')
    functions.add_argument('-e', '--exit', action='store_true',
                           required=False, dest='exit', default=False,
                           help='print error message and exit with status code -1')
    functions.add_argument('-n', '--nap', action='store_true',
                           required=False, dest='nap', default=False,
                           help='take a short nap')
    functions.add_argument('-d', '--dump', action='store_true',
                           required=False, dest='dump', default=False,
                           help='abort program and dump core')
    return parser

def main():
    arg_parser = make_arg_parser()
    args = arg_parser.parse_args()
    handler = LogHandler(profile=args.profile, application='null_executable')
    logger.addHandler(handler)

    # Shamelessly stolen from epilogue with a twist: allow for explict profile setting via the CL
    if 'CAPO_PROFILE' not in os.environ and '' == args.profile:
        # try to synthesize a profile from our installation root
        profile = os.path.abspath(sys.argv[0]).split(os.path.sep)[-3]
        os.environ['CAPO_PROFILE'] = profile
        print('No CAPO_PROFILE explicitly set, synthesizing {0} by path'.format(str(profile)))
    elif '' != args.profile:
        os.environ['CAPO_PROFILE'] = args.profile

    null = Null(args, args.verbose)
    null.execute()

if __name__ == '__main__':
    main()